@@ -1,27 +1,32 @@
-from __future__ import unicode_literals, absolute_import
+from __future__ import absolute_import, unicode_literals
 
 from unittest import TestCase as UnitTestCase
 
 import django
-from django.conf import settings
-from django.core.exceptions import ValidationError
+from django.contrib.contenttypes.models import ContentType
 from django.core import serializers
-from django.db import connection
+from django.core.exceptions import ImproperlyConfigured, ValidationError
 from django.test import TestCase, TransactionTestCase
-from django.utils import six
 from django.utils.encoding import force_text
 
-from django.contrib.contenttypes.models import ContentType
+from .forms import CustomPKFoodForm, DirectFoodForm, FoodForm, OfficialFoodForm
+from .models import (Article, Child, CustomManager, CustomPKFood,
+                     CustomPKHousePet, CustomPKPet, DirectFood,
+                     DirectHousePet, DirectPet, Food, HousePet, Movie,
+                     OfficialFood, OfficialHousePet, OfficialPet,
+                     OfficialTag, OfficialThroughModel, Pet, Photo,
+                     TaggedCustomPKFood, TaggedCustomPKPet, TaggedFood,
+                     TaggedPet)
 
-from taggit.managers import TaggableManager, _model_name
+from taggit.managers import _model_name, _TaggableManager, TaggableManager
 from taggit.models import Tag, TaggedItem
-from .forms import (FoodForm, DirectFoodForm, CustomPKFoodForm,
-    OfficialFoodForm)
-from .models import (Food, Pet, HousePet, DirectFood, DirectPet,
-    DirectHousePet, TaggedPet, CustomPKFood, CustomPKPet, CustomPKHousePet,
-    TaggedCustomPKPet, OfficialFood, OfficialPet, OfficialHousePet,
-    OfficialThroughModel, OfficialTag, Photo, Movie, Article)
-from taggit.utils import parse_tags, edit_string_for_tags
+
+from taggit.utils import edit_string_for_tags, parse_tags
+
+try:
+    from unittest import skipIf, skipUnless
+except ImportError:
+    from django.utils.unittest import skipIf, skipUnless
 
 
 class BaseTaggingTest(object):
@@ -83,6 +88,14 @@ class TagModelTestCase(BaseTaggingTransactionTestCase):
             "category-awesome-1"
         ], attr="slug")
 
+    def test_integers(self):
+        """Adding an integer as a tag should raise a ValueError (#237)."""
+        apple = self.food_model.objects.create(name="apple")
+        with self.assertRaisesRegexp(ValueError, (
+                r"Cannot add 1 \(<(type|class) 'int'>\). "
+                r"Expected <class 'django.db.models.base.ModelBase'> or str.")):
+            apple.tags.add(1)
+
 class TagModelDirectTestCase(TagModelTestCase):
     food_model = DirectFood
     tag_model = Tag
@@ -105,7 +118,7 @@ class TaggableManagerTestCase(BaseTaggingTestCase):
     def test_add_tag(self):
         apple = self.food_model.objects.create(name="apple")
         self.assertEqual(list(apple.tags.all()), [])
-        self.assertEqual(list(self.food_model.tags.all()),  [])
+        self.assertEqual(list(self.food_model.tags.all()), [])
 
         apple.tags.add('green')
         self.assert_tags_equal(apple.tags.all(), ['green'])
@@ -146,7 +159,7 @@ class TaggableManagerTestCase(BaseTaggingTestCase):
         #      make sure we don't double create.
         # + 12 on Django 1.6 for save points.
         queries = 22
-        if django.VERSION < (1,6):
+        if django.VERSION < (1, 6):
             queries -= 12
         self.assertNumQueries(queries, apple.tags.add, "red", "delicious", "green")
 
@@ -156,7 +169,7 @@ class TaggableManagerTestCase(BaseTaggingTestCase):
         #     make sure we dont't double create.
         # + 4 on Django 1.6 for save points.
         queries = 9
-        if django.VERSION < (1,6):
+        if django.VERSION < (1, 6):
             queries -= 4
         self.assertNumQueries(queries, pear.tags.add, "green", "delicious")
 
@@ -177,7 +190,7 @@ class TaggableManagerTestCase(BaseTaggingTestCase):
 
     def test_delete_bulk(self):
         apple = self.food_model.objects.create(name="apple")
-        kitty = self.pet_model.objects.create(pk=apple.pk,  name="kitty")
+        kitty = self.pet_model.objects.create(pk=apple.pk, name="kitty")
 
         apple.tags.add("red", "delicious", "fruit")
         kitty.tags.add("feline")
@@ -218,9 +231,9 @@ class TaggableManagerTestCase(BaseTaggingTestCase):
         pks = self.pet_model.objects.filter(tags__name__in=["fuzzy"])
         model_name = self.pet_model.__name__
         self.assertQuerysetEqual(pks,
-            ['<{0}: kitty>'.format(model_name),
-             '<{0}: cat>'.format(model_name)],
-            ordered=False)
+                                 ['<{0}: kitty>'.format(model_name),
+                                  '<{0}: cat>'.format(model_name)],
+                                 ordered=False)
 
     def test_lookup_bulk(self):
         apple = self.food_model.objects.create(name="apple")
@@ -250,14 +263,14 @@ class TaggableManagerTestCase(BaseTaggingTestCase):
         pear = self.food_model.objects.create(name="pear")
         pear.tags.add("green", "delicious")
 
-        guava = self.food_model.objects.create(name="guava")
+        self.food_model.objects.create(name="guava")
 
         pks = self.food_model.objects.exclude(tags__name__in=["red"])
         model_name = self.food_model.__name__
         self.assertQuerysetEqual(pks,
-            ['<{0}: pear>'.format(model_name),
-             '<{0}: guava>'.format(model_name)],
-            ordered=False)
+                                 ['<{0}: pear>'.format(model_name),
+                                  '<{0}: guava>'.format(model_name)],
+                                 ordered=False)
 
     def test_similarity_by_tag(self):
         """Test that pears are more similar to apples than watermelons"""
@@ -289,18 +302,17 @@ class TaggableManagerTestCase(BaseTaggingTestCase):
             '%s__name' % _model_name(self.pet_model): 'Spot'
         }
         self.assert_tags_equal(
-           self.tag_model.objects.filter(**lookup_kwargs),
-           ['scary']
+            self.tag_model.objects.filter(**lookup_kwargs),
+            ['scary']
         )
 
     def test_taggeditem_unicode(self):
-        ross = self.pet_model.objects.create(name="ross")
-        # I keep Ross Perot for a pet, what's it to you?
-        ross.tags.add("president")
+        apple = self.food_model.objects.create(name="apple")
+        apple.tags.add("juicy")
 
         self.assertEqual(
             force_text(self.taggeditem_model.objects.all()[0]),
-            "ross tagged with president"
+            "apple tagged with juicy"
         )
 
     def test_abstract_subclasses(self):
@@ -322,6 +334,7 @@ class TaggableManagerTestCase(BaseTaggingTestCase):
         # Check if tag field, which simulates m2m, has django-like api.
         field = self.food_model._meta.get_field('tags')
         self.assertTrue(hasattr(field, 'rel'))
+        self.assertTrue(hasattr(field.rel, 'to'))
         self.assertTrue(hasattr(field, 'related'))
         self.assertEqual(self.food_model, field.related.model)
 
@@ -355,18 +368,17 @@ class TaggableManagerTestCase(BaseTaggingTestCase):
                 'apple': set(['1', '2'])
             })
 
-
 class TaggableManagerDirectTestCase(TaggableManagerTestCase):
     food_model = DirectFood
     pet_model = DirectPet
     housepet_model = DirectHousePet
-    taggeditem_model = TaggedPet
+    taggeditem_model = TaggedFood
 
 class TaggableManagerCustomPKTestCase(TaggableManagerTestCase):
     food_model = CustomPKFood
     pet_model = CustomPKPet
     housepet_model = CustomPKHousePet
-    taggeditem_model = TaggedCustomPKPet
+    taggeditem_model = TaggedCustomPKFood
 
     def test_require_pk(self):
         # TODO with a charfield pk, pk is never None, so taggit has no way to
@@ -391,6 +403,16 @@ class TaggableManagerOfficialTestCase(TaggableManagerTestCase):
 
         self.assertEqual(apple, self.food_model.objects.get(tags__official=False))
 
+class TaggableManagerInitializationTestCase(TaggableManagerTestCase):
+    """Make sure manager override defaults and sets correctly."""
+    food_model = Food
+    custom_manager_model = CustomManager
+
+    def test_default_manager(self):
+        self.assertEqual(self.food_model.tags.__class__, _TaggableManager)
+
+    def test_custom_manager(self):
+        self.assertEqual(self.custom_manager_model.tags.__class__, CustomManager.Foo)
 
 class TaggableFormTestCase(BaseTaggingTestCase):
     form_class = FoodForm
@@ -479,7 +501,7 @@ class TagStringParseTestCase(UnitTestCase):
         self.assertEqual(parse_tags(',one two'), ['one two'])
         self.assertEqual(parse_tags(',one two three'), ['one two three'])
         self.assertEqual(parse_tags('a-one, a-two and a-three'),
-            ['a-one', 'a-two and a-three'])
+                         ['a-one', 'a-two and a-three'])
 
     def test_with_double_quoted_multiple_words(self):
         """
@@ -491,7 +513,7 @@ class TagStringParseTestCase(UnitTestCase):
         self.assertEqual(parse_tags('"one two three'), ['one', 'three', 'two'])
         self.assertEqual(parse_tags('"one two"'), ['one two'])
         self.assertEqual(parse_tags('a-one "a-two and a-three"'),
-            ['a-one', 'a-two and a-three'])
+                         ['a-one', 'a-two and a-three'])
 
     def test_with_no_loose_commas(self):
         """
@@ -510,9 +532,9 @@ class TagStringParseTestCase(UnitTestCase):
         Double quotes can contain commas
         """
         self.assertEqual(parse_tags('a-one "a-two, and a-three"'),
-            ['a-one', 'a-two, and a-three'])
+                         ['a-one', 'a-two, and a-three'])
         self.assertEqual(parse_tags('"two", one, one, two, "one"'),
-            ['one', 'two'])
+                         ['one', 'two'])
 
     def test_with_naughty_input(self):
         """
@@ -527,7 +549,7 @@ class TagStringParseTestCase(UnitTestCase):
         self.assertEqual(parse_tags(',,,,,,'), [])
         self.assertEqual(parse_tags('",",",",",",","'), [','])
         self.assertEqual(parse_tags('a-one "a-two" and "a-three'),
-            ['a-one', 'a-three', 'a-two', 'and'])
+                         ['a-one', 'a-three', 'a-two', 'and'])
 
     def test_recreation_of_tag_list_string_representations(self):
         plain = Tag.objects.create(name='plain')
@@ -538,3 +560,40 @@ class TagStringParseTestCase(UnitTestCase):
         self.assertEqual(edit_string_for_tags([plain, spaces, comma]), '"com,ma", "spa ces", plain')
         self.assertEqual(edit_string_for_tags([plain, comma]), '"com,ma", plain')
         self.assertEqual(edit_string_for_tags([comma, spaces]), '"com,ma", "spa ces"')
+
+
+@skipIf(django.VERSION < (1, 7), "not relevant for Django < 1.7")
+class DeconstructTestCase(UnitTestCase):
+    def test_deconstruct_kwargs_kept(self):
+        instance = TaggableManager(through=OfficialThroughModel, to='dummy.To')
+        name, path, args, kwargs = instance.deconstruct()
+        new_instance = TaggableManager(*args, **kwargs)
+        self.assertEqual('tests.OfficialThroughModel', new_instance.rel.through)
+        self.assertEqual('dummy.To', new_instance.rel.to)
+
+
+@skipUnless(django.VERSION < (1, 7), "test only applies to 1.6 and below")
+class SouthSupportTests(TestCase):
+    def test_import_migrations_module(self):
+        try:
+            from taggit.migrations import __doc__  # noqa
+        except ImproperlyConfigured as e:
+            exception = e
+        self.assertIn("SOUTH_MIGRATION_MODULES", exception.args[0])
+
+
+class InheritedPrefetchTests(TestCase):
+
+    def test_inherited_tags_with_prefetch(self):
+        child = Child()
+        child.save()
+        child.tags.add('tag 1', 'tag 2', 'tag 3', 'tag 4')
+
+        child = Child.objects.get()
+        no_prefetch_tags = child.tags.all()
+        self.assertEquals(4, no_prefetch_tags.count())
+        child = Child.objects.prefetch_related('tags').get()
+        prefetch_tags = child.tags.all()
+        self.assertEquals(4, prefetch_tags.count())
+        self.assertEquals(set([t.name for t in no_prefetch_tags]),
+                          set([t.name for t in prefetch_tags]))
